from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .widgets import MapInput, AddressAutoHiddenInput


def reverse_tuple_string(location_string):
    if location_string == "":
        return location_string
    args = location_string.split(",")
    return args[1] + "," + args[0]


def parse_location(location_string, first_in_order="lng"):
    """parse and convert coordinates from string to tuple"""

    args = location_string.split(",")
    if len(args) != 2:
        raise ValidationError(_("Invalid input for a Location instance"))

    if first_in_order == "lat":
        lng = args[1]
        lat = args[0]
    elif first_in_order == 'lng':
        lng = args[0]
        lat = args[1]
    else:
        raise ValidationError(_("Invalid first letter for parsing location."))
    try:
        lat = float(lat)
    except ValueError:
        raise ValidationError(_("Invalid input for a Location instance. Latitude must be convertible to float "))
    try:
        lng = float(lng)
    except ValueError:
        raise ValidationError(_("Invalid input for a Location instance. Longitude must be convertible to float "))

    return lng, lat


class LocationField(forms.CharField):
    """custom form field for picking location"""

    def __init__(self, *args, **kwargs):
        map_attrs = kwargs.pop("map_attrs", None)
        self.widget = MapInput(map_attrs=map_attrs, )

        super().__init__(*args, **kwargs)
        self.error_messages = {"required": "Please pick a location, it's required", }

    def to_python(self, value):
        return super().to_python(reverse_tuple_string(value))


class AddressAutoHiddenField(forms.CharField):
    """custom form field which uses AddressAutoHiddenInput"""
    widget = AddressAutoHiddenInput()

    def __init__(self, *args, **kwargs):
        map_id = kwargs.pop("map_id", "map")
        self.widget = AddressAutoHiddenInput(map_id=map_id)
        super().__init__(*args, **kwargs)
        self.label = ""



from django.contrib.gis.forms import PointField, ValidationError
from django.contrib.gis.geos import Point

from ..forms import parse_location
from ..widgets import MapInput


class SpatialLocationField(PointField):
    """custom form field for picking location for spatial databases"""

    def __init__(self, *args, **kwargs):
        map_attrs = kwargs.pop("map_attrs", None)
        self.widget = MapInput(map_attrs=map_attrs, )

        super().__init__(*args, **kwargs)
        self.error_messages = {"required": "Please pick a location, it's required", }

    def clean(self, value):
        try:
            return super().clean(value)
        except (ValueError, ValidationError):
            return None

    def to_python(self, value):
        """Transform the value to a Geometry object."""
        if value in self.empty_values:
            return None

        if isinstance(value, Point):
            return value

        return Point(parse_location(value, first_in_order="lat"), srid=4326)



from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def transform_map_attrs_to_js(id, key, map_attr):
    """renders javascript variable from python attribute"""
    js_pattern = "map_attrs['{id}'].{key} = ".format(id=id, key=key)
    if type(map_attr) == list or type(map_attr) == tuple:
        return mark_safe(js_pattern + str(list(map_attr)) + ";")
    elif type(map_attr) == bool:
        return mark_safe(js_pattern + str(map_attr).lower() + ";")
    else:
        return mark_safe(js_pattern + "'{str}'".format(str=map_attr) + ";")



from django.conf import settings
from django.forms import Media
from django.forms.widgets import TextInput


def parse_tuple_string(tuple_string):
    if "POINT" in tuple_string:
        list = tuple_string.split(" ")[1:]
        list[0] = list[0][1:]
        list[1] = list[1][:-1]
        return tuple(map(float, list))

    return tuple(map(float, tuple_string[1:-1].split(",")))


class MapInput(TextInput):
    """widget for picking your location"""

    template_name = "mapbox_location_field/map_input.html"

    def __init__(self, attrs=None, map_attrs=None):
        if map_attrs is None:
            map_attrs = {}
        self.map_attrs = map_attrs
        self.readonly = self.map_attrs.pop("readonly", True)
        self.placeholder = self.map_attrs.pop("placeholder", "Pick a location on map below")
        self.center_point = False

        super().__init__(attrs)

    class Media:
        js = ("mapbox_location_field/js/map_input.js",)
        css = {
            "all": ("mapbox_location_field/css/map_input.css",)
        }

    def get_context(self, name, value, attrs):
        if attrs is None:
            attrs = {}

        must_be_attrs = {
            "readonly": self.readonly,
            "placeholder": self.placeholder
        }
        attrs.update(must_be_attrs)
        attrs["class"] = attrs.get("class", "") + " js-mapbox-input-location-field"
        attrs["id"] = self.map_attrs.get("id", "map")

        context = super().get_context(name, value, attrs)
        self.center_point = context["widget"].get("value", False)
        context["key"] = settings.MAPBOX_KEY
        context["mapbox_attrs"] = self.get_config_settings()
        return context

    def get_config_settings(self):
        """renders javascript configuration variables definitions"""
        default_map_attrs = {
            "style": "mapbox://styles/mapbox/outdoors-v11",
            "zoom": 13,
            "center": [17.031645, 51.106715],
            "cursor_style": 'pointer',
            "marker_color": "red",
            "rotate": False,
            "geocoder": True,
            "fullscreen_button": True,
            "navigation_buttons": True,
            "track_location_button": True,
            "id": "map"
        }

        if self.map_attrs is not None:
            default_map_attrs.update(self.map_attrs)

        if self.center_point:
            default_map_attrs["center"] = parse_tuple_string(self.center_point)

        return default_map_attrs


class AddressAutoHiddenInput(TextInput):
    """hidden text input which automatically fill itself with address from MapInput"""
    template_name = "mapbox_location_field/address_input.html"

    def __init__(self, attrs=None, map_id="map"):
        self.map_id = map_id
        super().__init__(attrs)
        self.label = ""

    def get_context(self, name, value, attrs):
        if attrs is None:
            attrs = {}
        attrs["class"] = attrs.get("class", "") + " js-mapbox-address-input-location-field"
        context = super().get_context(name, value, attrs)

        context["map_id"] = self.map_id
        return context

    class Media:
        js = ("mapbox_location_field/js/address_input.js",)
        css = {
            "all": ("mapbox_location_field/css/address_input.css",)
        }


class MapAdminInput(MapInput):
    """map input, but with javascript excluded from media"""

    @property
    def media(self):
        return Media(css={"all": ("mapbox_location_field/css/map_input.css",)})


class AddressHiddenAdminInput(AddressAutoHiddenInput):
    """address input, but with javascript excluded from media"""

    @property
    def media(self):
        return Media(css={"all": ("mapbox_location_field/css/address_input.css",)})
